#include <amxmodx>
#include <amxmisc>
#include <colorchat2>

#define PLUGIN "Support"
#define VERSION "1.0"
#define AUTHOR "Maka"

#define MAX_SUPPORT_REQUESTS 32
#define ADMIN_ACCESS ADMIN_CHAT
#define SUPPORT_LIBRARY "support"
#define LOGS_FOLDER "addons/amxmodx/configs/Support"

enum _:SupportData {
    SUPPORT_AUTHOR[32],
    SUPPORT_STEAMID[35],
    SUPPORT_MESSAGE[192],
    SUPPORT_TIME[32],
    bool:SUPPORT_ACTIVE,
    SUPPORT_USERID   
}

new g_SupportRequests[MAX_SUPPORT_REQUESTS][SupportData]
new g_TotalRequests = 0
new g_CurrentRequest[33]
new g_MenuPage[33]

public plugin_natives() {
    register_library(SUPPORT_LIBRARY)
    register_native("ShowSupport", "native_show_support")
}

public plugin_init() {
    register_plugin(PLUGIN, VERSION, AUTHOR)
    
    register_clcmd("say /support", "CmdSupport")
    register_clcmd("say /sa", "CmdSupportAdmin")
    register_clcmd("SUPPORT_MESSAGE", "HandleSupportMessage")
    register_clcmd("ADMIN_RESPONSE", "HandleAdminResponse")
    
    register_menu("SupportMenu", 1023, "HandleSupportMenu")
    register_menu("RequestDetails", 1023, "HandleRequestDetails")
    
    register_event("HLTV", "EventNewRound", "a", "1=0", "2=0")
    
    
    
    // Initialize menu pages for all player slots
    for(new i = 0; i < 33; i++) {
        g_MenuPage[i] = 0
    }
}


stock fmt(const format[], any:...)
{
    static buffer[1024]
    vformat(buffer, sizeof(buffer) - 1, format, 2)
    return buffer
}

stock LogSupportAction(const type[], const player[], const admin[] = "", const message[] = "") {
    new logFile[96], date[32]
    get_time("%Y-%m-%d", date, charsmax(date))
    formatex(logFile, charsmax(logFile), "%s/support_log_%s.txt", LOGS_FOLDER, date)
    
    new timeStr[32]
    get_time("%H:%M:%S", timeStr, charsmax(timeStr))
    
    if(equal(type, "request")) {
        log_to_file(logFile, "[%s] New support request from %s: %s", timeStr, player, message)
    }
    else if(equal(type, "answer")) {
        log_to_file(logFile, "[%s] Admin %s answered to %s: %s", timeStr, admin, player, message)
    }
    else if(equal(type, "solved")) {
        log_to_file(logFile, "[%s] Admin %s marked request from %s as SOLVED", timeStr, admin, player)
    }
    else if(equal(type, "delete")) {
        log_to_file(logFile, "[%s] Admin %s deleted request from %s", timeStr, admin, player)
    }
}

public native_show_support(plugin_id, param_count) {
    new id = get_param(1)
    if(is_user_connected(id) && access(id, ADMIN_ACCESS)) {
        ShowSupportMenu(id)
        return 1
    }
    return 0
}

public CmdSupport(id) {
    if(!is_user_connected(id)) return PLUGIN_HANDLED
    
    client_cmd(id, "messagemode SUPPORT_MESSAGE")
    client_print_color(id, "!g[McDonald's Support]!y Please enter your support message:")
    
    return PLUGIN_HANDLED
}

public HandleSupportMessage(id) {
    if(!is_user_connected(id)) return PLUGIN_HANDLED
    
    new message[192]
    read_args(message, 191)
    remove_quotes(message)
    
    if(strlen(message) < 1) return PLUGIN_HANDLED
    
    AddSupportRequest(id, message)
    client_print_color(id, "^4[Support]^1 Your support request has been submitted!")
    
    NotifyAdmins()
    
    return PLUGIN_HANDLED
}


public CmdSupportAdmin(id) {
    if(!is_user_connected(id) || !access(id, ADMIN_ACCESS)) return PLUGIN_HANDLED
    
    ShowSupportMenu(id)
    return PLUGIN_HANDLED
}

ShowSupportMenu(id) {
    static menu[512], len, keys
    new start_index = g_MenuPage[id] * 7
    new end_index = start_index + 7
    new items_shown = 0
    new active_requests = 0
    
   
    for(new i = 0; i < MAX_SUPPORT_REQUESTS; i++) {
        if(g_SupportRequests[i][SUPPORT_ACTIVE]) {
            active_requests++
        }
    }
    
    keys = MENU_KEY_0
    len = formatex(menu, charsmax(menu), "\ySupport Menu \r[Page %d]^n^n", g_MenuPage[id] + 1)
    
    
    for(new i = start_index; i < end_index && i < MAX_SUPPORT_REQUESTS; i++) {
        if(g_SupportRequests[i][SUPPORT_ACTIVE]) {
            len += formatex(menu[len], charsmax(menu) - len, "\r%d. \w%s \y[%s]^n", 
                items_shown + 1, 
                g_SupportRequests[i][SUPPORT_AUTHOR], 
                g_SupportRequests[i][SUPPORT_TIME])
            keys |= (1<<items_shown)
            items_shown++
        }
    }
    
    
    len += formatex(menu[len], charsmax(menu) - len, "^n")
    
    
    if(active_requests > (g_MenuPage[id] + 1) * 7) {
        len += formatex(menu[len], charsmax(menu) - len, "\r8. \wNext Page^n")
        keys |= MENU_KEY_8
    }
    
   
    if(g_MenuPage[id] > 0) {
        len += formatex(menu[len], charsmax(menu) - len, "\r9. \wPrevious Page^n")
        keys |= MENU_KEY_9
    }
    
    len += formatex(menu[len], charsmax(menu) - len, "^n\r0. \wExit")
    
    show_menu(id, keys, menu, -1, "SupportMenu")
    return PLUGIN_HANDLED
}


public HandleSupportMenu(id, key) {
    switch(key) {
        case 7: {
            g_MenuPage[id]++
            ShowSupportMenu(id)
            return PLUGIN_HANDLED
        }
        case 8: { 
            if(g_MenuPage[id] > 0) {
                g_MenuPage[id]--
                ShowSupportMenu(id)
            }
            return PLUGIN_HANDLED
        }
        case 9: { 
            return PLUGIN_HANDLED
        }
        default: {
            
            new actual_index = (g_MenuPage[id] * 7) + key
            if(g_SupportRequests[actual_index][SUPPORT_ACTIVE]) {
                ShowRequestDetails(id, actual_index)
            }
        }
    }
    return PLUGIN_HANDLED
}


ShowRequestDetails(id, request_id) {
    static menu[512], len, keys
    
    keys = MENU_KEY_1|MENU_KEY_2|MENU_KEY_3|MENU_KEY_0
    
    len = formatex(menu, charsmax(menu), "\yRequest Details^n^n")
    len += formatex(menu[len], charsmax(menu) - len, "\wPlayer: %s^n", g_SupportRequests[request_id][SUPPORT_AUTHOR])
    len += formatex(menu[len], charsmax(menu) - len, "SteamID: %s^n", g_SupportRequests[request_id][SUPPORT_STEAMID])
    len += formatex(menu[len], charsmax(menu) - len, "Message: %s^n^n", g_SupportRequests[request_id][SUPPORT_MESSAGE])
    
    len += formatex(menu[len], charsmax(menu) - len, "\r1. \wAnswer^n")
    len += formatex(menu[len], charsmax(menu) - len, "\r2. \wMark as SOLVED^n")
    len += formatex(menu[len], charsmax(menu) - len, "\r3. \wDelete^n")
    len += formatex(menu[len], charsmax(menu) - len, "^n\r0. \wBack")
    
    g_CurrentRequest[id] = request_id
    show_menu(id, keys, menu, -1, "RequestDetails")
}

public HandleRequestDetails(id, key) {
    new request_id = g_CurrentRequest[id]
    
    switch(key) {
        case 0: { // Answer
            client_cmd(id, "messagemode ADMIN_RESPONSE")
        }
        case 1: { // Mark as SOLVED
            MarkRequestSolved(request_id, id)
            ShowSupportMenu(id)
        }
        case 2: { // Delete
            DeleteRequest(request_id, id)
            ShowSupportMenu(id)
        }
        case 9: { // Back
            ShowSupportMenu(id)
        }
    }
    return PLUGIN_HANDLED
}

MarkRequestSolved(request_id, id) {
    if(!g_SupportRequests[request_id][SUPPORT_ACTIVE])
        return
   
    new players[32], num
    get_players(players, num)
    
    for(new i = 0; i < num; i++) {
        new player = players[i]
        new player_name[32]
        get_user_name(player, player_name, 31)
        
       
        if(equal(player_name, g_SupportRequests[request_id][SUPPORT_AUTHOR])) {
          
            client_print_color(player, "!g[Support]!y Your request has been marked as !tSOLVED")
            client_print_color(player, "!g[Support]!y Thank you for contacting our support team!")
            
            new admin_name[32]
            get_user_name(id, admin_name, 31)
            LogSupportAction("solved", g_SupportRequests[request_id][SUPPORT_AUTHOR], admin_name)  // Add this line here
            
            break
        }
    }
    
    DeleteRequest(request_id, id)
}



DeleteRequest(request_id, id) {
    new admin_name[32]
    get_user_name(id, admin_name, 31)
    LogSupportAction("delete", g_SupportRequests[request_id][SUPPORT_AUTHOR], admin_name)  
    
    g_SupportRequests[request_id][SUPPORT_ACTIVE] = false
    g_SupportRequests[request_id][SUPPORT_AUTHOR][0] = 0
    g_SupportRequests[request_id][SUPPORT_STEAMID][0] = 0
    g_SupportRequests[request_id][SUPPORT_MESSAGE][0] = 0
    g_SupportRequests[request_id][SUPPORT_TIME][0] = 0
}


public HandleAdminResponse(id) {
    if(!is_user_connected(id)) return PLUGIN_HANDLED
    
    new response[192], admin_name[32]
    read_args(response, 191)
    remove_quotes(response)
    get_user_name(id, admin_name, 31)
    
    
    new players[32], num
    get_players(players, num)
    
    for(new i = 0; i < num; i++) {
        new player = players[i]
        new player_name[32]
        get_user_name(player, player_name, 31)
        
        
        if(equal(player_name, g_SupportRequests[g_CurrentRequest[id]][SUPPORT_AUTHOR])) {
            
            client_print_color(player,  "!g[Support]!y Admin !t%s!y responded to your request:", admin_name)
            client_print_color(player,  "!g[Support]!y Message: !t%s", response)
            
            LogSupportAction("answer", g_SupportRequests[g_CurrentRequest[id]][SUPPORT_AUTHOR], admin_name, response)  
            
            break
        }
    }
    
    client_print_color(id, "!g[Support]!y Your response has been sent to !t%s", g_SupportRequests[g_CurrentRequest[id]][SUPPORT_AUTHOR])
    
    ShowSupportMenu(id)
    return PLUGIN_HANDLED
}


public EventNewRound() {
    for(new i = 0; i < MAX_SUPPORT_REQUESTS; i++) {
        g_SupportRequests[i][SUPPORT_ACTIVE] = false
    }
    g_TotalRequests = 0
}

AddSupportRequest(id, const message[]) {
    new name[32], steamid[35], time[32]
    get_user_name(id, name, 31)
    get_user_authid(id, steamid, 34)
    get_time("%H:%M:%S", time, 31)
    
    new request_id = g_TotalRequests
    copy(g_SupportRequests[request_id][SUPPORT_AUTHOR], 31, name)
    copy(g_SupportRequests[request_id][SUPPORT_STEAMID], 34, steamid)
    copy(g_SupportRequests[request_id][SUPPORT_MESSAGE], 191, message)
    copy(g_SupportRequests[request_id][SUPPORT_TIME], 31, time)
    g_SupportRequests[request_id][SUPPORT_ACTIVE] = true
    g_SupportRequests[request_id][SUPPORT_USERID] = get_user_userid(id)
    
    LogSupportAction("request", name, "", message)  
    
    g_TotalRequests++
}
NotifyAdmins() {
    new players[32], pnum
    get_players(players, pnum, "ch")
    
    for(new i = 0; i < pnum; i++) {
        if(access(players[i], ADMIN_ACCESS)) {
            client_print_color(players[i], "!g[Support]!t New support request received!")
            set_task(0.1, "ShowMenuToAdmin", players[i])  
        }
    }
}

public ShowMenuToAdmin(id) {
    if(is_user_connected(id) && access(id, ADMIN_ACCESS)) {
        ShowSupportMenu(id)
    }
}
